#!/usr/bin/env python

"""
Parsing of textual content.

Copyright (C) 2016 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re

# Parsing of lines to obtain functions and arguments.

line_pattern_str = r"(?:" \
                   r"(?:'(.*?)')" \
                   r"|" \
                   r'(?:"(.*?)")' \
                   r"|" \
                   r"([^\s]+)" \
                   r")+" \
                   r"(?:\s+|$)"
line_pattern = re.compile(line_pattern_str)

def parse_line(text):

    """
    Parse the given 'text', returning a list of words separated by whitespace in
    the input, where whitespace may occur inside words if quoted using single or
    double quotes.
    """

    parts = []

    # Match the components of each part.

    for match in line_pattern.finditer(text):

        # Combine the components by traversing the matching groups.

        parts.append(reduce(lambda a, b: (a or "") + (b or ""), match.groups()))

    return parts

# vim: tabstop=4 expandtab shiftwidth=4
